#include "ushell.h"

; The quick little program to patch the jp table according to rom version.
; must be assembled in put into a string WITHOUT a zshell program header
; so use:
;     tasmb -80 -g3 -q -r8 patchrom.asm
;     rename patchrom.obj patchrom.bin
;     bin2str patchrom.bin /H /N:patchrom

; all other programs for UShell should use String85 from ZShell 3.1 to be
; converted to a string.  It expects the same sort of header as Zshell programs.
; Patchrom doesn't have that header so it doesn't show up on the program 
; selection menu.

JumpOffset = 08c97h

.org 0

	ld a, (ROM_VERS)
	cp 057h
	jr z, ROM30a
	cp 08eh
	jr z, ROM40
	cp 03dh
	jr z, ROM60
	cp 097h
	jr z, ROM80
	ld a, (03c80h)
	cp 016h
	jr z, ROM90
	cp 0cdh
	jr z, ROM10

	ret

ROM40:
ROM60:
	ld de, 3
	jr CopyOldPatches
ROM80:
	ld de, 7
	jr CopyOldPatches
ROM30a:
	ld de, 0
CopyOldPatches:
	ld (JumpOffset), de
	ld hl, (PROGRAM_ADDR)
	ld de, OLDROMPATCH
	add hl, de
	ld c, (ROM10PATCHe - ROM10PATCHs)/3
	ld de, 08c0ch  	; Set destination
AdjustNext:
	push bc
	ld a, (hl)
	inc hl
	ld (de), a
	inc de
	ld c, (hl)
	inc hl
	ld b, (hl)
	inc hl

	push hl
	ld hl, (JumpOffset)
	add hl, bc
	ex de, hl
	ld (hl), e
	inc hl
	ld (hl), d
	inc hl
	ex de, hl
	pop hl
	pop bc
	dec c
	jr nz, AdjustNext
	ret

ROM90:
	ld hl, ROM9PATCHs
	jr CopyWithoutAdj
ROM10:
	ld hl, ROM10PATCHs
CopyWithoutAdj:
	ld de, (PROGRAM_ADDR)
	add hl, de
	ld de, 08c0ch
	ld bc, ROM10PATCHe-ROM10PATCHs
	ldir
	ret

OLDROMPATCH:
	jp 03be0h
	jp 03c22h
	jp 03bdah
	jp 03d90h
	jp 03e5ch
	jp 03dfch
	jp 03e02h
	jp 03e68h
	jp 03b9eh
	jp 03df6h
	jp 03feeh
	jp 03ebch
	jp 03ea4h
	jp 039e8h

ROM9PATCHs:
	jp 03b9fh
	jp 03be1h
	jp 0
	jp 0
	jp 03e39h
	jp 03dd3h
	jp 03dd9h
	jp 03e45h
	jp 03b5dh
	jp 0
	jp 0
	jp 03e93h
	jp 03e7bh
	jp 0

ROM10PATCHs:
	jp 03bbah
	jp 03bfch
	jp 03bb4h
	jp 0
	jp 03e54h
	jp 03deeh
	jp 03df4h
	jp 03e60h
	jp 03b78h
	jp 03fe0h
	jp 03fe0h
	jp 03eaeh
	jp 03e96h
	jp 0
ROM10PATCHe:

.end
