; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
; The assembly source code to a ZShell-like machine code loader
; (Works with ROM's 9 and 10)
; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
; Copyright 1995, Philip Gray Montgomery
; any comments/ideas/etc can be sent to:
; pgm@cgmsun.physics.utoledo.edu
; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

; features are less then Zshell, but it does have the following advantages:
; fast rom calls - Rom calls are actually a call into a jump table.  No
;		 overhead from indexing and all calls are one byte shorter
;		 because it only uses a single call unlike Zshell's call
;		 followed by rom index number
; Allows for future rom - when the current version of rom doesn't match
; 		 the version of rom it remembers being last run with, then
;		 it searches memory for the string, "patchrom" and then
;		 executes it.  patchrom takes care of all of the work involved
;		 with figuring out what the addresses, and since it's a string
;		 like everything else, can be replaced with a newer version
;		 without having to restore a patched backup.
; Smaller - This probably has to do with the many fewer features then Zshell
;		 but it IS smaller.  ;)  Anyway, you can even delete patchrom
;		 if you know you're not going to copy it to another calculator.
;		 I think Ushell is something like 400-500 bytes...

; Ushell is pretty worthless now that Zshell supports the new rom, but
; I thought someone out there might be interested in it.

; rom entry points for rom v8
rD_LT_STR	=$3B9E+7	; display length-byte normal text string
rM_CHARPUT	=$3BDA+7 ; display %A on screen, menu style
rD_ZM_STR	=$3BE0+7	; display zero-terminated string, menu style
rD_LM_STR	=$3C22+7	; display length-byte string, menu style
rGET_T_CUR	=$3CA0+7	; HL = absolute address of text cursor
rSCROLL_UP	=$3D90+7	; scroll text screen up
rTR_CHARPUT	=$3DF6+7	; raw display of %A on screen, normal text style
rCLEARLCD	=$3DFC+7	; clear LCD, but not text or graphics memory
rD_HL_DECI	=$3E02+7	; disp. HL as 5-byte, right-just., blank-pad. decimal
rCLEARTEXT	=$3E5C+7	; clear LCD and text memory (affected by 1,(IY+13))
rD_ZT_STR	=$3E68+7	; display zero-terminated normal text string
rBUSY_OFF	=$3EA4+7	; turn off "busy" indicator
rBUSY_ON	=$3EBC+7	; turn on "busy" indicator
rTX_CHARPUT	=$3FEE+7	; xlated display of %A on screen, normal text style
rRANDOM		=$39E8+7	; make a random number

#include "ushell.h"
; All programs that use ushell need to include ushell.h for the macros
; defined.

; Must be patched in at 8c00h
; how you get it their is an interesting question, but it's possible.  ;)
.org 8c00h

.dw Start			; ptr to execution start
.db "UShell", 0

RomVersion:
.db 97h
LoadedProgAddr:
.dw 0

ROM_ENTRIES_START:
	jp rD_ZM_STR
	jp rD_LM_STR
	jp rM_CHARPUT
	jp rSCROLL_UP
	jp rCLEARTEXT
	jp rCLEARLCD
	jp rD_HL_DECI
	jp rD_ZT_STR
	jp rD_LT_STR
	jp rTR_CHARPUT
	jp rTX_CHARPUT
	jp rBUSY_ON
	jp rBUSY_OFF
	jp rRANDOM
ROM_ENTRIES_END:

extrn_call_:
	jr _call
	nop
	nop
extrn_call_z:
	jr z, _call
	jr _conditionfail
extrn_call_nz:
	jr nz, _call
	jr _conditionfail
extrn_call_c:
	jr c, _call
	jr _conditionfail
extrn_call_nc:
	jr nc, _call
	jr _conditionfail

extrn_jump_:
	jr _jump
	nop
	nop
extrn_jump_z:
	jr z, _jump
	jr _conditionfail
extrn_jump_nz:
	jr nz, _jump
	jr _conditionfail
extrn_jump_c:
	jr c, _jump
	jr _conditionfail
extrn_jump_nc:
	jr nc, _jump
	jr _conditionfail

_call:
	ld (Saved_hl), hl
	ld (Saved_bc), bc
	pop hl		; hl -> return address
	ld c, (hl)
	inc hl
	ld b, (hl)
	inc hl		; load address into bc
	push hl		; store return ptr
	ld hl, (LoadedProgAddr)
	add hl, bc
	push hl
	ld bc, (Saved_bc)
	ld hl, (Saved_hl)
	ret

_jump:
	ld (Saved_hl), hl
	ld (Saved_bc), bc
	pop hl
	ld c, (hl)
	inc hl
	ld b, (hl)
	ld hl, (LoadedProgAddr)
	add hl, bc
	push hl
	ld hl, (Saved_hl)
	ld bc, (Saved_bc)
	ret

_conditionfail:
	ex (sp), hl
	inc hl
	inc hl			; Skip over address
	ex (sp), hl
	ret

Saved_hl:
.dw 0
Saved_bc:
.dw 0


TitleLine:
#ifdef ROM_10
.db "UShell 10.0 ROM ALPHA", 0
#else
#ifdef ROM_9
.db "UShell  9.0 ROM ALPHA", 0
#else
.db "UShell Cool ROM ALPHA", 0
#endif
#endif
;    012345678901234567890
ExitMsg1:
.db "~~~~~~~~~~~~~~~~~~~~~",0
ExitMsg2:
.db "UShell Version 0.1",0
ExitMsg3:
.db "Copr 1995, PGM",0
NoProgramMsg:
.db "** No More Programs **", 0
NameBuffer:
.db 0,0,0,0,0,0,0,0,0,0
ProgramTable:
.dw 0,0,0,0,0,0,0,0
BarPos:
.db 0
LastProgram:
.dw 0
TotalEntries:
.db 0
Start:
	in a, (5)
	push af			; save current selected rom page

	ld hl, (0834bh)		; save printing params
	push hl
	ld hl, (08353h)
	push hl
	ld hl, (0800ch)
	push hl
	ld hl, (08333h)
	push hl

	ld a, 7
	out (5), a		; Select page 7 for ROM version number
	ld a, (RomVersion)
	ld b, a
	ld a, (07fffh)
	cp b
	call nz, LoadNewROMMap

	CALL_ROM(BUSY_OFF)

	ld a, 0
	ld (08353h), a

InitData:
	call ZeroMem
	ld hl, VAT_START-1-1-6-2
	ld (LastProgram), hl

	; Basics done, starting menu -------------------------

DrawMainScr:
	sub a
	ld (BarPos), a

	CALL_ROM(CLEARLCD)
	sub a
	ld (CURSOR_ROW), a
	ld (CURSOR_COL), a
	ld hl, TitleLine
	CALL_ROM(D_ZT_STR)

	ld a, 1
	ld (CURSOR_X), a
	ld (CURSOR_Y), a

	ld hl, (LastProgram)
	ld de, ProgramTable
	ld c, 8

AdvanceWord:
	ld a, 1
	ld (CURSOR_X), a
	ld a, (CURSOR_Y)
	add a, 7
	ld (CURSOR_Y), a

	jr TestWord

NextWord:
	push bc
	ld a, b
	neg
	ld c, a
	ld b, -1
	add hl, bc
	pop bc

TestWord:
;	push bc
	ld a, (VAT_END+1)
	cp h
	jr c, Bigger
	ld a, (VAT_END)
	cp l
;	pop bc
	jr nc, RanOutofProgs	; Check to see if entire memory scanned

Bigger:
	ld a, (hl)
	dec hl
	dec hl
	dec hl			; point hl to name length
	ld b, (hl)
	dec hl			; Points to name now
	cp 00ch
	jr nz, NextWord
	call IsValidProg
	
	jr nc, NextWord

	push hl
	push de
	ld de, NameBuffer
NextNameChar:
	ld a, (hl)
	ld (de), a
	dec hl
	inc de
	dec b
	jr nz, NextNameChar
	sub a
	ld (de), a		; Copy name into ASCIIZ string

	pop de
	ex (sp), hl		; save ptr to next word and
				; get start of name ptr
	push hl
	ld hl, NameBuffer
	CALL_ROM(D_ZM_STR)	; print it
	pop hl			; Get back the ptr to name
	
	push bc
	inc hl			; hl -> length
	inc hl			; hl -> address high
	ld b, (hl)
	inc hl			; hl -> addr low
	ld c, (hl)
	inc bc			; skip length
	inc bc
	inc bc			; skip null
	inc bc
	inc bc			; skip checksum

	ld a, c			; store low
	ld (de), a
	inc de
	ld a, b			; store high
	ld (de), a
	inc de

	ld h, b
	ld l, c
	pop bc

	ld a, 40
	ld (CURSOR_X), a
	CALL_ROM(D_ZM_STR)		; print description

	pop hl			; restore pointer to next word
	dec c
	jr nz, AdvanceWord

	ld (LastProgram), hl
	jr DoneWordScan

RanOutofProgs:
NextSpaceToNull:
	ld hl, NoProgramMsg
	CALL_ROM(D_ZM_STR)		; print it
	sub a
	ld (de), a
	inc de
	ld (de), a
	ld hl, VAT_START-1-1-6-2
	ld (LastProgram), hl

DoneWordScan:
	ld a, 8
	sub c
	ld (TotalEntries), a
	; Wait for user input
	jr FirstDraw

UpdateScr:
	call DrawSelect
	ld a, (BarPos)
	add a, b
	bit 7, a			; is it negative
	jr z, NotNegative
	ld a, (TotalEntries)
	dec a
NotNegative:
	ld b, a
	ld a, (TotalEntries)
	cp b
	jr nz, SetBarPos
	ld b, 0
SetBarPos:
	ld a, b
	ld (BarPos), a
FirstDraw:
	call DrawSelect

PollKey:
	call GET_KEY
	cp 37h
	jr z, ExitShell
	cp 1
	jr nz, NotDown
	ld b, 1
	jr UpdateScr
NotDown:
	cp 4
	jr nz, NotUp
	ld b, -1
	jr UpdateScr

NotUp:
	cp 038h
	jr nz, NotMore
	jp DrawMainScr
NotMore:
	cp 009h
	jr nz, NotEnter
	call ExecuteCommand			; error here??
	jp InitData
NotEnter:
	jp PollKey


ExitShell:
	set 1, (iy+0dh)
	CALL_ROM(CLEARTEXT)
	ld hl, 0005h
	ld (CURSOR_ROW), hl
	ld hl, ExitMsg1
	CALL_ROM(D_ZT_STR)
	ld hl, 0006h
	ld (CURSOR_ROW), hl
	ld hl, ExitMsg2
	CALL_ROM(D_ZT_STR)
	ld hl, 0007h
	ld (CURSOR_ROW), hl
	ld hl, ExitMsg3
	CALL_ROM(D_ZT_STR)

	sub a
	ld (CURSOR_ROW), a
	ld (CURSOR_COL), a

	pop hl
	ld hl, (08333h)
	pop hl
	ld hl, (0800ch)
	pop hl
	ld hl, (08353h)
	pop hl
	ld hl, (0834bh)
	pop af
	out (5), a		; restore rom page

	jp 462h			; return to normal execution??

IsValidProg:			; hl -> name of vat entry
;	ccf
;	ret			; well, like ya know...

	push hl
	push de
	inc hl			; skip name len
	inc hl			; ptr to addr
	ld d, (hl)
	inc hl
	ld e, (hl)
	ld a, (de)
	ld h, a
	inc de
	ld a, (de)
	or h
	jr z, NotValidProg
Nottoosmall:
	inc de			; skip past length
	ld a, (de)		; Check first two bytes of data
	or a
	jr nz, NotValidProg	; if first byte is not a null
___Vv:
	scf
	jr ReturnValidRes
NotValidProg:
	scf
	ccf			; set cy to zero
ReturnValidRes:
	pop de
	pop hl
	ret

DrawSelect:
	ld a, (TotalEntries)
	or a
	ret z
	push bc
	push hl

	ld a, (BarPos)
	add a, a	;*2
	add a, a	;*4
	ld h, 0
	ld l, a
	add hl, hl	;*8
	add hl, hl	;*16
	ld b, h
	ld c, l
	add hl, hl	;
	add hl, hl	;
	add hl, bc	;
	add hl, bc	;
	add hl, bc	;
	ld bc, VIDEO_MEM
	add hl, bc
	ld bc, 8*16
	add hl, bc
	ld bc, 16*7

dog:
	ld a, (hl)
	cpl
	ld (hl), a
	inc hl
	dec bc
	ld a, c
	or b
	jr nz, dog

	pop hl
	pop bc
	ret

ExecuteCommand:
	push hl
	push de
	ld a, (BarPos)
	add a, a
	ld h, 0
	ld l, a
	ld de, ProgramTable
	add hl, de
	ld de, NextOpCode
	push de
	ld e, (hl)
	inc hl
	ld d, (hl)
	ld (LoadedProgAddr), de

nextnamechar:
	ld a, (de)
	or a
	jr z, EndofName
	inc de
	jr nextnamechar

EndofName:
	inc de			; jump past last null
	push de
	call ZeroMem
	ret
NextOpCode:
	pop de
	pop hl
	ret
	
Waitforkey:
	call GET_KEY
	cp 37h
	ret z
	jr Waitforkey

ZeroMem:
	ld hl, 080dfh
	ld de, 080e0h
	ld bc, 000a7h
	ld (hl), 0		; Zero out first byte of text mem
	ldir
	ret

_patchmap
.db "patchrom"

LoadNewROMMap:	; a -> rom id
	ld (RomVersion), a
	ld hl, _patchmap
	ld de, 08084h
	ld bc, 8
	ldir
	ld a, 0ch
	ld (08082h), a
	ld a, 8
	ld (08083h), a
	rst 10h			; search for it
	ret c
	ex de, hl		; hl -> patchRom
	inc hl
	inc hl			; -> to patch map
	ld (LoadedProgAddr), hl
	push hl
	ret			; Execute (RET will return to previous call)
.end
