#DEFINE CALL_ROM(addr)		call addr
#DEFINE CALL_(addr)		call $8C36 \ .dw addr
#DEFINE CALL_Z(addr)		call $8C3a \ .dw addr
#DEFINE CALL_NZ(addr)		call $8C3e \ .dw addr
#DEFINE CALL_C(addr)		call $8C42 \ .dw addr
#DEFINE CALL_NC(addr)		call $8C46 \ .dw addr

#DEFINE JUMP_(addr)		call $8C4a \ .dw addr
#DEFINE JUMP_Z(addr)		call $8C4e \ .dw addr
#DEFINE JUMP_NZ(addr)		call $8C52 \ .dw addr
#DEFINE JUMP_C(addr)		call $8C56 \ .dw addr
#DEFINE JUMP_NC(addr)		call $8C5a \ .dw addr

;
; UShell Specifics
;

PROGRAM_ADDR	=$8C0a	; program address
ROM_VERS	=$8C09	; ROM version byte

;
; Call Addresses (based on ROM 3.0A)
;
LD_HL_MHL	=$0033	; HL=(HL), A=L
CP_HL_DE	=$008E	; CP HL,DE (modifies flags only)
UNPACK_HL	=$009A	; unpacks one digit of HL into %A
STORE_KEY	=$01B1	; store immediate keystroke from %A
GET_KEY		=$01BE	; get immediate keystroke into %A

; ROM subroutines
D_LT_STR	=08c24h	; display length-byte normal text string
M_CHARPUT	=08c12h	; display %A on screen, menu style
D_ZM_STR	=08c0ch	; display zero-terminated string, menu style
D_LM_STR	=08c0fh	; display length-byte string, menu style
SCROLL_UP	=08c15h	; scroll text screen up
TR_CHARPUT	=08c27h	; raw display of %A on screen, normal text style
CLEARLCD	=08c1bh	; clear LCD, but not text or graphics memory
D_HL_DECI	=08c1eh	; disp. HL as 5-byte, right-just., blank-pad. decimal
CLEARTEXT	=08c18h	; clear LCD and text memory (affected by 1,(IY+13))
D_ZT_STR	=08c21h	; display zero-terminated normal text string
BUSY_OFF	=08c30h	; turn off "busy" indicator
BUSY_ON		=08c2dh	; turn on "busy" indicator
TX_CHARPUT	=08c2ah	; xlated display of %A on screen, normal text style
RANDOM		=08c33h	; make a random number

;
; Data Locations
;
KEY_0		=$8000	; translated scancode of last key, but 0 if gotten
KEY_1		=$8001	; translated scancode of key down now
KEY_2		=$8002	; same as 8001, but $FF if more than one key is down
KEY_STAT	=$8004	; bit 2 set = key down now
LAST_KEY	=$8006	; last key pressed
CONTRAST	=$8007	; contrast
CURSOR_ROW	=$800C	; text cursor row
CURSOR_COL	=$800D	; text cursor column
CURSOR_LET	=$800E	; letter under text cursor
BUSY_COUNTER	=$8080	; counter for busy indicator
BUSY_BITMAP	=$8081	; bitmap for busy indicator
CURR_INPUT	=$80C6	; -> current home-screen input
BYTES_USED	=$80CC	; # of used user memory (Add to 80C8 to find first 
			; byte of free memory)
TEXT_MEM	=$80DF	; text memory
CURSOR_ROW2	=$800C	; text cursor row
CURSOR_COL2	=$800D	; text cursor column
CHECKSUM	=$81BE	; memory checksum from 8BF7 to FA6F
CURSOR_X	=$8333	; x value of cursor
CURSOR_Y	=$8334	; y value of cursor
IY_TABLE	=$8346	; where IY usually points
GRAPH_MEM	=$8641	; graphics memory
TEXT_MEM2	=$8A6B	; secondary text memory
USER_MEM	=$8B1B	; -> start of user memory
;=$8B1D	; subtract (8B1B) from this to get # of free bytes
FIXED_POINT	=$8B3A	; fixed-point decimal place (FFh for floating point)
VAT_END		=$8BEB	; -> one byte before end of VAT (backwards)
VAT_START	=$FA6F	; start of VAT
VIDEO_MEM	=$FC00	; video memory
